﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using Telerik.Windows.Controls;

namespace Grouping
{
    public class GroupingFilteringBehavior : FilteringBehavior
    {
        public override IEnumerable<object> FindMatchingItems(string searchText, IList items, IEnumerable<object> escapedItems, string textSearchPath, TextSearchMode textSearchMode)
        {
            return string.IsNullOrEmpty(searchText) ? Enumerable.Empty<object>() : this.ExpandControls(items, searchText);
        }

        private IEnumerable<object> ExpandControls(IList items, string searchText)
        {
            List<Group> matchedControls = new List<Group>();
            List<Item> matchedExamples = new List<Item>();
            IEnumerable<Item> examples;
            List<object> result = new List<object>();

            foreach (var item in items.OfType<Group>())
            {
                if (item.Name.StartsWith(searchText, StringComparison.InvariantCultureIgnoreCase))
                {
                    if (!matchedControls.Contains(item))
                    {
                        matchedControls.Add(item);
                        result.Add(item);
                    }

                    foreach (var example in item.Items)
                    {
                        if (!matchedExamples.Contains(example))
                        {
                            matchedExamples.Add(example);
                            result.Add(example);
                        }
                    }
                }
                else
                {
                    examples = item.Items.Where(e => e.Name.StartsWith(searchText, StringComparison.InvariantCultureIgnoreCase));

                    if (examples.Any())
                    {
                        if (!matchedControls.Contains(item))
                        {
                            matchedControls.Add(item);
                            result.Add(item);
                        }
                    }

                    foreach (var example in examples)
                    {
                        if (!matchedExamples.Contains(example))
                        {
                            matchedExamples.Add(example);
                            result.Add(example);
                        }
                    }
                } 
            }

            Group lastGroup = null;

            foreach (var item in result)
            {
                if (item is Group)
                {
                    lastGroup = item as Group;
                    lastGroup.Number = 0;
                }
                else
                {
                    lastGroup.Number++;
                }
            }

            return result;
        }
    }
}
